mod migration;
mod migrations;
mod program;
mod section;

pub use migration::*;
pub use migrations::create_migrations;

pub use rusqlite::Connection as DbConn;

use sprinklers_core::model::{Program, Programs, Sections};

use eyre::Result;
use rusqlite::{params, NO_PARAMS};

pub fn setup_db() -> Result<DbConn> {
    // let conn = DbConn::open_in_memory()?;
    let mut conn = DbConn::open("test.db")?;

    let migs = create_migrations();
    migs.apply(&mut conn)?;

    Ok(conn)
}

pub fn query_sections(conn: &DbConn) -> Result<Sections> {
    let mut statement = conn.prepare_cached(
        "SELECT s.id, s.name, s.interface_id \
        FROM sections AS s;",
    )?;
    let rows = statement.query_map(NO_PARAMS, section::from_sql)?;
    let mut sections = Sections::new();
    for row in rows {
        let section = row?;
        sections.insert(section.id, section.into());
    }
    Ok(sections)
}

pub fn query_programs(conn: &DbConn) -> Result<Programs> {
    let mut statement = conn.prepare_cached(
        "
        SELECT p.id, p.name, p.enabled, p.schedule, ps.sequence
        FROM programs AS p
        INNER JOIN program_sequences AS ps
            ON ps.program_id = p.id;",
    )?;
    let rows = statement.query_map(NO_PARAMS, program::from_sql)?;
    let mut programs = Programs::new();
    for row in rows {
        let program = row?;
        programs.insert(program.id, program.into());
    }
    Ok(programs)
}

pub fn update_program(conn: &mut DbConn, prog: &Program) -> Result<()> {
    let trans = conn.transaction()?;
    trans
        .prepare_cached(
            "
        UPDATE programs
            SET (name, enabled, schedule) = (?2, ?3, ?4)
            WHERE id = ?1;",
        )?
        .execute(&program::as_sql(prog))?;
    trans
        .prepare_cached(
            "
        DELETE FROM program_sequence_items AS psi
            WHERE psi.program_id = ?1;",
        )?
        .execute(params![prog.id])?;
    let mut insert_prog_seq = trans.prepare_cached(
        "
        INSERT INTO program_sequence_items
            (program_id, seq_num, section_id, duration)
            VALUES (?1, ?2, ?3, ?4);",
    )?;
    for params in program::sequence_as_sql(prog) {
        insert_prog_seq.execute(&params)?;
    }
    drop(insert_prog_seq);
    trans.commit()?;
    Ok(())
}
